@@ -360,20 +360,22 @@ IDirect3DTextureImpl_Load(IDirect3DTexture2 *iface,
             if(ret_value != D3D_OK)
             {
                 ERR("IWineD3DPalette::GetParent failed! This is unexpected\n");
+                if (pal) IDirectDrawPalette_Release(pal);
                 LeaveCriticalSection(&ddraw_cs);
                 return D3DERR_TEXTURE_LOAD_FAILED;
             }
         }
 
-        /* After seeing some logs, not sure at all about this... */
-        if (pal == NULL)
-        {
-            IWineD3DSurface_SetPalette(This->WineD3DSurface, wine_pal);
-            if (pal_src != NULL) IDirectDrawPalette_AddRef(pal_src);
-        }
-        else if (pal_src != NULL)
+        if (pal_src != NULL)
         {
             PALETTEENTRY palent[256];
+
+            if (pal == NULL)
+            {
+                IDirectDrawPalette_Release(pal_src);
+                LeaveCriticalSection(&ddraw_cs);
+                return DDERR_NOPALETTEATTACHED;
+            }
             IDirectDrawPalette_GetEntries(pal_src, 0, 0, 256, palent);
             IDirectDrawPalette_SetEntries(pal, 0, 0, 256, palent);
         }
