@@ -74,7 +74,7 @@ const GUID DInput_Wine_Mouse_GUID = { /* 9e573ed8-7734-11d2-8d4a-23903fb6bdf7 */
     0x9e573ed8, 0x7734, 0x11d2, {0x8d, 0x4a, 0x23, 0x90, 0x3f, 0xb6, 0xbd, 0xf7}
 };
 
-void _dump_mouse_state(DIMOUSESTATE2 *m_state)
+static void _dump_mouse_state(DIMOUSESTATE2 *m_state)
 {
     int i;
 
@@ -301,7 +301,7 @@ static void dinput_mouse_hook( LPDIRECTINPUTDEVICE8A iface, WPARAM wparam, LPARA
                 /* Already have X, need to queue it */
                 if (inst_id != -1)
                     queue_event((LPDIRECTINPUTDEVICE8A)This, id_to_offset(&This->base.data_format, inst_id),
-                                wdata, hook->time, This->base.dinput->evsequence);
+                                wdata, GetCurrentTime(), This->base.dinput->evsequence);
                 inst_id = DIDFT_MAKEINSTANCE(WINE_MOUSE_Y_AXIS_INSTANCE) | DIDFT_RELAXIS;
                 wdata = pt1.y;
             }
@@ -352,7 +352,7 @@ static void dinput_mouse_hook( LPDIRECTINPUTDEVICE8A iface, WPARAM wparam, LPARA
     {
         _dump_mouse_state(&This->m_state);
         queue_event((LPDIRECTINPUTDEVICE8A)This, id_to_offset(&This->base.data_format, inst_id),
-                    wdata, hook->time, This->base.dinput->evsequence++);
+                    wdata, GetCurrentTime(), This->base.dinput->evsequence++);
     }
 
     LeaveCriticalSection(&This->base.crit);
