@@ -309,9 +309,9 @@ HRESULT IDirectMusicUtils_IPersistStream_ParseReference (LPPERSISTSTREAM iface,
     }
     TRACE_(dmfile)(": ListCount[0] = %d < ListSize[0] = %d\n", ListCount[0], ListSize[0]);
   } while (ListCount[0] < ListSize[0]);
-  
+
   ref_desc.dwValidData |= DMUS_OBJ_CLASS;
-  memcpy(&ref_desc.guidClass, &ref.guidClassID, sizeof(ref.guidClassID));
+  ref_desc.guidClass = ref.guidClassID;
 
   TRACE_(dmfile)("** DM Reference Begin of Load ***\n");
   TRACE_(dmfile)("With Desc:\n");
