@@ -54,6 +54,12 @@ typedef struct tagInputContextData
         INPUTCONTEXT    IMC;
 } InputContextData;
 
+typedef struct _tagTRANSMSG {
+    UINT message;
+    WPARAM wParam;
+    LPARAM lParam;
+} TRANSMSG, *LPTRANSMSG;
+
 static InputContextData *root_context = NULL;
 static HWND hwndDefault = NULL;
 static HANDLE hImeInst;
@@ -188,23 +194,33 @@ static LRESULT ImmInternalSendIMENotify(WPARAM notify, LPARAM lParam)
     return 0;
 }
 
+static HIMCC ImmCreateBlankCompStr(void)
+{
+    HIMCC rc;
+    LPCOMPOSITIONSTRING ptr;
+    rc = ImmCreateIMCC(sizeof(COMPOSITIONSTRING));
+    ptr = (LPCOMPOSITIONSTRING)ImmLockIMCC(rc);
+    memset(ptr,0,sizeof(COMPOSITIONSTRING));
+    ptr->dwSize = sizeof(COMPOSITIONSTRING);
+    ImmUnlockIMCC(rc);
+    return rc;
+}
+
 static void ImmInternalSetOpenStatus(BOOL fOpen)
 {
     TRACE("Setting internal state to %s\n",(fOpen)?"OPEN":"CLOSED");
 
-   root_context->IMC.fOpen = fOpen;
-   root_context->bInternalState = fOpen;
-
-   if (fOpen == FALSE)
-   {
+    if (root_context->IMC.fOpen && fOpen == FALSE)
+    {
         ShowWindow(hwndDefault,SW_HIDE);
         ImmDestroyIMCC(root_context->IMC.hCompStr);
-        root_context->IMC.hCompStr = NULL;
+        root_context->IMC.hCompStr = ImmCreateBlankCompStr();
     }
-    else
-        ShowWindow(hwndDefault, SW_SHOWNOACTIVATE);
 
-   ImmInternalSendIMENotify(IMN_SETOPENSTATUS, 0);
+    root_context->IMC.fOpen = fOpen;
+    root_context->bInternalState = fOpen;
+
+    ImmInternalSendIMENotify(IMN_SETOPENSTATUS, 0);
 }
 
 static int updateField(DWORD origLen, DWORD origOffset, DWORD currentOffset,
@@ -593,6 +609,9 @@ HIMC WINAPI ImmCreateContext(void)
 
     new_context = HeapAlloc(GetProcessHeap(),HEAP_ZERO_MEMORY,sizeof(InputContextData));
 
+    /* hCompStr is never NULL */
+    new_context->IMC.hCompStr = ImmCreateBlankCompStr();
+
     return (HIMC)new_context;
 }
 
@@ -1407,14 +1426,17 @@ BOOL WINAPI ImmNotifyIME(
                     TRACE("%s - %s\n","NI_COMPOSITIONSTR","CPS_CANCEL");
                     {
                         BOOL send;
+                        LPCOMPOSITIONSTRING lpCompStr;
 
                         if (pX11DRV_ForceXIMReset)
                             pX11DRV_ForceXIMReset(root_context->IMC.hWnd);
 
-                        send = (root_context->IMC.hCompStr!=NULL);
+                        lpCompStr = ImmLockIMCC(root_context->IMC.hCompStr);
+                        send = (lpCompStr->dwCompStrLen != 0);
+                        ImmUnlockIMCC(root_context->IMC.hCompStr);
 
                         ImmDestroyIMCC(root_context->IMC.hCompStr);
-                        root_context->IMC.hCompStr = NULL;
+                        root_context->IMC.hCompStr = ImmCreateBlankCompStr();
 
                         if (send)
                             ImmInternalPostIMEMessage(WM_IME_COMPOSITION, 0,
@@ -1995,6 +2017,33 @@ DWORD WINAPI ImmGetIMCCSize(HIMCC imcc)
     return internal->dwSize;
 }
 
+/***********************************************************************
+*		ImmGenerateMessage(IMM32.@)
+*/
+BOOL WINAPI ImmGenerateMessage(HIMC hIMC)
+{
+    InputContextData *data = (InputContextData*)hIMC;
+
+    TRACE("%i messages queued\n",data->IMC.dwNumMsgBuf);
+    if (data->IMC.dwNumMsgBuf > 0)
+    {
+        LPTRANSMSG lpTransMsg;
+        INT i;
+
+        lpTransMsg = (LPTRANSMSG)ImmLockIMCC(data->IMC.hMsgBuf);
+        for (i = 0; i < data->IMC.dwNumMsgBuf; i++)
+            ImmInternalPostIMEMessage(lpTransMsg[i].message, lpTransMsg[i].wParam, lpTransMsg[i].lParam);
+
+        ImmUnlockIMCC(data->IMC.hMsgBuf);
+        ImmDestroyIMCC(data->IMC.hMsgBuf);
+
+        data->IMC.dwNumMsgBuf = 0;
+        data->IMC.hMsgBuf = NULL;
+    }
+
+    return TRUE;
+}
+
 /*****
  * Internal functions to help with IME window management
  */
