@@ -15,7 +15,7 @@
  *
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 #include "msvcrt.h"
 #include "wine/debug.h"
@@ -71,7 +71,7 @@ static DWORD CALLBACK _beginthread_trampoline(LPVOID arg)
 /*********************************************************************
  *		_beginthread (MSVCRT.@)
  */
-MSVCRT_uintptr_t _beginthread(
+MSVCRT_uintptr_t CDECL _beginthread(
   MSVCRT__beginthread_start_routine_t start_address, /* [in] Start address of routine that begins execution of new thread */
   unsigned int stack_size, /* [in] Stack size for new thread or 0 */
   void *arglist)           /* [in] Argument list to be passed to new thread or NULL */
@@ -96,7 +96,7 @@ MSVCRT_uintptr_t _beginthread(
 /*********************************************************************
  *		_beginthreadex (MSVCRT.@)
  */
-MSVCRT_uintptr_t _beginthreadex(
+MSVCRT_uintptr_t CDECL _beginthreadex(
   void *security,          /* [in] Security descriptor for new thread; must be NULL for Windows 9x applications */
   unsigned int stack_size, /* [in] Stack size for new thread or 0 */
   MSVCRT__beginthreadex_start_routine_t start_address, /* [in] Start address of routine that begins execution of new thread */
@@ -108,14 +108,14 @@ MSVCRT_uintptr_t _beginthreadex(
 
   /* FIXME */
   return (MSVCRT_uintptr_t)CreateThread(security, stack_size,
-				     (LPTHREAD_START_ROUTINE) start_address,
-				     arglist, initflag, (LPDWORD) thrdaddr);
+				     start_address, arglist,
+				     initflag, thrdaddr);
 }
 
 /*********************************************************************
  *		_endthread (MSVCRT.@)
  */
-void _endthread(void)
+void CDECL _endthread(void)
 {
   TRACE("(void)\n");
 
@@ -126,7 +126,7 @@ void _endthread(void)
 /*********************************************************************
  *		_endthreadex (MSVCRT.@)
  */
-void _endthreadex(
+void CDECL _endthreadex(
   unsigned int retval) /* [in] Thread exit code */
 {
   TRACE("(%d)\n", retval);
