@@ -375,6 +375,23 @@ static void test_begin_paint(void)
         "clip box should have been reset %d,%d-%d,%d\n", rect.left, rect.top, rect.right, rect.bottom );
 }
 
+static void test_invisible_create(void)
+{
+    HWND hwnd_owndc = CreateWindowA("owndc_class", NULL, WS_OVERLAPPED,
+                                    0, 200, 100, 100,
+                                    0, 0, GetModuleHandleA(0), NULL );
+    HDC dc1, dc2;
+
+    dc1 = GetDC(hwnd_owndc);
+    dc2 = GetDC(hwnd_owndc);
+
+todo_wine
+    ok(dc1 == dc2, "expected owndc dcs to match\n");
+
+    ReleaseDC(hwnd_owndc, dc2);
+    ReleaseDC(hwnd_owndc, dc1);
+    DestroyWindow(hwnd_owndc);
+}
 
 START_TEST(dce)
 {
@@ -414,4 +431,5 @@ START_TEST(dce)
     test_parameters();
     test_dc_visrgn();
     test_begin_paint();
+    test_invisible_create();
 }
