@@ -78,7 +78,8 @@ static inline int last_array(const type_t *type)
 
 static inline int is_string_type(const attr_list_t *attrs, const type_t *type)
 {
-    return is_attr(attrs, ATTR_STRING) && (last_ptr(type) || last_array(type));
+    return ((is_attr(attrs, ATTR_STRING) || is_attr(type->attrs, ATTR_STRING))
+            && (last_ptr(type) || last_array(type)));
 }
 
 static inline int is_context_handle(const type_t *type)
